@@ -27,6 +27,8 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#include <unistd.h>
+
 #include <glib/gprintf.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
@@ -66,20 +68,18 @@ _gs_lock_plug_profile_log (const char *func,
                            const char *msg1,
                            const char *msg2)
 {
-        char    *str;
-        GTimeVal now;
+        char *str;
 
         if (indent < 0)
                 profile_add_indent (indent);
 
-        g_get_current_time (&now);
-
         if (profile_indent == 0)
-                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+                str = g_strdup_printf ("MARK: %s: %s %s %s", G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
         else
-                str = g_strdup_printf ("MARK %ld.%6.6ld: %s: %*c %s %s %s", now.tv_sec, now.tv_usec, G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+                str = g_strdup_printf ("MARK: %s: %*c %s %s %s", G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+
+        access (str, F_OK);
 
-        fprintf (stderr, "%s\n", str);
         g_free (str);
 
         if (indent > 0)
@@ -131,6 +131,7 @@ struct GSLockPlugPrivate
         gboolean     caps_lock_on;
         gboolean     switch_enabled;
         gboolean     logout_enabled;
+        char        *logout_command;
 
         guint        timeout;
 
@@ -157,6 +158,7 @@ enum {
 enum {
         PROP_0,
         PROP_LOGOUT_ENABLED,
+        PROP_LOGOUT_COMMAND,
         PROP_SWITCH_ENABLED
 };
 
@@ -265,6 +267,14 @@ set_status_text (GSLockPlug *plug,
         gtk_label_set_text (GTK_LABEL (plug->priv->status_label), text);
 }
 
+static void
+set_dialog_sensitive (GSLockPlug *plug,
+                      gboolean    sensitive)
+{
+        gtk_widget_set_sensitive (plug->priv->password_entry, sensitive);
+        gtk_widget_set_sensitive (plug->action_area, sensitive);
+}
+
 static void
 gs_lock_plug_response (GSLockPlug *plug,
                        gint        response_id)
@@ -304,7 +314,7 @@ gs_lock_plug_response (GSLockPlug *plug,
                 gint current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
 
                 if (current_page == 0) {
-                        gtk_widget_set_sensitive (plug->priv->password_entry, FALSE);
+                        set_dialog_sensitive (plug, FALSE);
                         set_status_text (plug, _("Checking password..."));
 
                         plug->priv->password_check_idle_id = g_idle_add ((GSourceFunc)password_check_idle_cb,
@@ -349,12 +359,12 @@ monitor_progress (GSLockPlug *plug)
 
         if ((remaining <= 0) || (remaining > plug->priv->timeout)) {
                 message = g_strdup (_("Time has expired."));
-                gtk_widget_set_sensitive (plug->priv->password_entry, FALSE);
+                set_dialog_sensitive (plug, FALSE);
                 set_status_text (plug, message);
                 g_free (message);
 
                 if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (1000,
+                        plug->priv->response_idle_id = g_timeout_add (2000,
                                                                       (GSourceFunc)response_idle_cb,
                                                                       plug);
                 return FALSE;
@@ -431,6 +441,22 @@ gs_lock_plug_hide (GtkWidget *widget)
                 GTK_WIDGET_CLASS (parent_class)->hide (widget);
 }
 
+
+static void
+gs_lock_plug_size_request (GtkWidget      *widget,
+                           GtkRequisition *requisition)
+{
+        if (GTK_WIDGET_CLASS (parent_class)->size_request)
+                GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+
+        /* Force width to be at least 1.2 times the height.
+         * This ensures enough space for the text entry and
+         * makes for a more pleasing aspect ratio */
+        if (requisition->width < requisition->height) {
+                requisition->width = requisition->height * 1.2;
+        }
+}
+
 static void
 gs_lock_plug_set_logout_enabled (GSLockPlug *plug,
                                  gboolean    logout_enabled)
@@ -449,6 +475,21 @@ gs_lock_plug_set_logout_enabled (GSLockPlug *plug,
                 gtk_widget_hide (plug->priv->logout_button);
 }
 
+static void
+gs_lock_plug_set_logout_command (GSLockPlug *plug,
+                                 const char *command)
+{
+        g_return_if_fail (GS_LOCK_PLUG (plug));
+
+        g_free (plug->priv->logout_command);
+
+        if (command) {
+                plug->priv->logout_command = g_strdup (command);
+        } else {
+                plug->priv->logout_command = NULL;
+        }
+}
+
 static void
 gs_lock_plug_set_switch_enabled (GSLockPlug *plug,
                                  gboolean    switch_enabled)
@@ -481,6 +522,9 @@ gs_lock_plug_set_property (GObject            *object,
         case PROP_LOGOUT_ENABLED:
                 gs_lock_plug_set_logout_enabled (self, g_value_get_boolean (value));
                 break;
+        case PROP_LOGOUT_COMMAND:
+                gs_lock_plug_set_logout_command (self, g_value_get_string (value));
+                break;
         case PROP_SWITCH_ENABLED:
                 gs_lock_plug_set_switch_enabled (self, g_value_get_boolean (value));
                 break;
@@ -504,6 +548,9 @@ gs_lock_plug_get_property (GObject    *object,
         case PROP_LOGOUT_ENABLED:
                 g_value_set_boolean (value, self->priv->logout_enabled);
                 break;
+        case PROP_LOGOUT_COMMAND:
+                g_value_set_string (value, self->priv->logout_command);
+                break;
         case PROP_SWITCH_ENABLED:
                 g_value_set_boolean (value, self->priv->switch_enabled);
                 break;
@@ -525,9 +572,10 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
         object_class->get_property = gs_lock_plug_get_property;
         object_class->set_property = gs_lock_plug_set_property;
 
-        widget_class->style_set  = gs_lock_plug_style_set;
-        widget_class->show       = gs_lock_plug_show;
-        widget_class->hide       = gs_lock_plug_hide;
+        widget_class->style_set    = gs_lock_plug_style_set;
+        widget_class->show         = gs_lock_plug_show;
+        widget_class->hide         = gs_lock_plug_hide;
+        widget_class->size_request = gs_lock_plug_size_request;
 
         g_type_class_add_private (klass, sizeof (GSLockPlugPrivate));
 
@@ -548,6 +596,13 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                NULL,
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_LOGOUT_COMMAND,
+                                         g_param_spec_string ("logout-command",
+                                                               NULL,
+                                                               NULL,
+                                                               NULL,
+                                                               G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_SWITCH_ENABLED,
                                          g_param_spec_boolean ("switch-enabled",
@@ -581,11 +636,14 @@ password_check_idle_cb (GSLockPlug *plug)
                                GS_LOCK_PLUG_RESPONSE_OK);
         } else {
                 if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (1000,
+                        plug->priv->response_idle_id = g_timeout_add (4000,
                                                                       (GSourceFunc)response_idle_cb,
                                                                       plug);
 
                 set_status_text (plug, _("That password was incorrect."));
+
+                printf ("NOTICE=AUTH FAILED\n");
+                fflush (stdout);
         }
 
         memset (local_password, '\b', strlen (local_password));
@@ -694,22 +752,32 @@ static void
 logout_button_clicked (GtkButton  *button,
                        GSLockPlug *plug)
 {
-        char   *argv [4];
-        GError *error = NULL;
+        char   **argv  = NULL;
+        GError  *error = NULL;
+        gboolean res;
 
-        argv [0] = BINDIR "/gnome-session-save";
-        argv [1] = "--kill";
-        argv [2] = "--silent";
-        argv [3] = NULL;
+        if (! plug->priv->logout_command)
+                return;
+
+        res = g_shell_parse_argv (plug->priv->logout_command, NULL, &argv, &error);
+
+        if (! res) {
+                g_warning ("Could not parse logout command: %s", error->message);
+                g_error_free (error);
+                return;
+        }
 
         g_spawn_async (g_get_home_dir (),
                        argv,
                        NULL,
-                       0,
+                       G_SPAWN_SEARCH_PATH,
                        NULL,
                        NULL,
                        NULL,
                        &error);
+
+        g_strfreev (argv);
+
         if (error) {
                 g_warning ("Could not run logout command: %s", error->message);
                 g_error_free (error);
@@ -741,15 +809,6 @@ entry_key_press (GtkWidget   *widget,
         if (capslock_on != plug->priv->caps_lock_on)
                 capslock_update (plug, capslock_on);
 
-        switch (event->keyval) {
-        case GDK_Return:
-        case GDK_KP_Enter:
-                gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_OK);
-                break;
-        default:
-                break;
-        }
-        
         return FALSE;
 }
 
@@ -1553,8 +1612,10 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         gtk_widget_show_all (plug->vbox);
 
-        if (! plug->priv->logout_enabled)
+        if (! plug->priv->logout_enabled
+            || ! plug->priv->logout_command)
                 gtk_widget_hide (plug->priv->logout_button);
+
         if (! plug->priv->switch_enabled)
                 gtk_widget_hide (plug->priv->switch_button);
 
@@ -1581,6 +1642,8 @@ gs_lock_plug_finalize (GObject *object)
 
         g_return_if_fail (plug->priv != NULL);
 
+        g_free (plug->priv->logout_command);
+
         if (plug->priv->fusa_manager)
                 g_object_unref (plug->priv->fusa_manager);
 
