@@ -68,8 +68,12 @@ void        gs_manager_set_lock_timeout     (GSManager  *manager,
                                              glong       lock_timeout);
 void        gs_manager_set_logout_enabled   (GSManager  *manager,
                                              gboolean    logout_enabled);
+void        gs_manager_set_user_switch_enabled (GSManager  *manager,
+                                                gboolean    user_switch_enabled);
 void        gs_manager_set_logout_timeout   (GSManager  *manager,
                                              glong       logout_timeout);
+void        gs_manager_set_logout_command   (GSManager  *manager,
+                                             const char *command);
 void        gs_manager_set_throttle_enabled (GSManager  *manager,
                                              gboolean    lock_enabled);
 void        gs_manager_set_cycle_timeout    (GSManager  *manager,
@@ -78,6 +82,7 @@ void        gs_manager_set_themes           (GSManager  *manager,
                                              GSList     *themes);
 void        gs_manager_set_mode             (GSManager  *manager,
                                              GSSaverMode mode);
+void        gs_manager_request_unlock       (GSManager  *manager);
 
 G_END_DECLS
 
